﻿Shader "NPR/ContoursLaplace"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;

			float4 frag (v2f i) : SV_Target
			{
				float L = dot(tex2D(_MainTex, i.uv).rgb, float3(0.2126, 0.7152, 0.0722));
				return float4(L,L,L,1);
			}
			ENDCG
		}

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

				struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}

			sampler2D _MainTex;
			sampler2D ColorTex;
			float4 _MainTex_TexelSize;
			float LaplaceThreshold;
			float LaplaceFalloff;
			float gradientThreshold;
			float gradientFalloff;
			float sigma;

			//Laplacian of Gaussian
			float LoG(float d, float sigma)
			{
				float t = -(d*d) / (2 * sigma*sigma);
				return -1.0 / (sigma*sigma*sigma*sigma*3.14) * (1.0+t) * exp(t);
			}

			float4 frag(v2f i) : SV_Target
			{
				float Lapl;
				if (sigma == 0)
				{
					Lapl = tex2D(_MainTex, i.uv + _MainTex_TexelSize * float2(1, 0)).r
						+ tex2D(_MainTex, i.uv + _MainTex_TexelSize * float2(0, 1)).r
						+ tex2D(_MainTex, i.uv + _MainTex_TexelSize * float2(-1, 0)).r
						+ tex2D(_MainTex, i.uv + _MainTex_TexelSize * float2(0, -1)).r
						- 4 * tex2D(_MainTex, i.uv).r;
				}
				else
				{
					float3 SUM = float3(0, 0, 0);
					float SUMW = 0;
					for (int iy = -5; iy <= 5; iy++)
						for (int ix = -5; ix <= 5; ix++)
						{
							float2 uv = i.uv + _MainTex_TexelSize.xy * float2(ix, iy);
							float3 col = tex2D(_MainTex, uv).rgb;
							float w = LoG(length((i.uv - uv) / _MainTex_TexelSize.xy), sigma);
							SUMW += w;
							SUM += col*w;
						}
					SUM /= SUMW;
					Lapl = SUM;
				}
				float edgeValue = 1.0 - smoothstep(LaplaceThreshold, LaplaceThreshold + LaplaceFalloff, abs(Lapl));
				float gradX = tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2( 1, 0)).r
					        - tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2(-1, 0)).r;
				float gradY = tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2( 0, 1)).r
					        - tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2( 0,-1)).r;
				float gradMag = length(float2(gradX, gradY));
				edgeValue *= smoothstep(max(0,gradientThreshold-gradientFalloff), gradientThreshold + gradientFalloff, gradMag);
				edgeValue = 1.0 - edgeValue;
				float4 imageColor = tex2D(ColorTex, i.uv);
				imageColor.rgb *= edgeValue;
				return imageColor;
			}
			ENDCG
		}
	}
}
