﻿// Upgrade NOTE: replaced 'mul(UNITY_MATRIX_MVP,*)' with 'UnityObjectToClipPos(*)'

Shader "NPR/ToonShaderWithContours"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
		contourThreshold ("Contour Threshold", Range(0.0,1.0)) = 0.3
		contourFalloff("Contour Falloff", Range(0.0,1.0)) = 0.05
	}
	SubShader
	{
		Tags { "RenderType"="Opaque" "LightMode"="ForwardBase" }
		LOD 100

		Pass
		{			
			CGPROGRAM
			#pragma vertex vert_toon
			#pragma fragment frag_toon

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float3 normal : NORMAL;
			};

			struct v2f_toon
			{
				float3 normal : NORMAL;
				float3 viewDir : TEXCOORD0;
				float3 lightDir : TEXCOORD1;
				float  CDepth : TEXCOORD2;
				float4 vertex : SV_POSITION;
			};

			sampler2D _MainTex;
			
			v2f_toon vert_toon(appdata v)
			{
				v2f_toon o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.normal = UnityObjectToWorldNormal(v.normal);
				o.viewDir = WorldSpaceViewDir(v.vertex);
				o.lightDir = WorldSpaceLightDir(v.vertex);
				o.CDepth = UnityObjectToViewPos(v.vertex).z;
				return o;
			}
			
			float contourThreshold;
			float contourFalloff;
			float4 frag_toon(v2f_toon i) : SV_Target
			{
				float3 N = normalize(i.normal);
				float3 L = normalize(i.lightDir);
				float NdL = dot(N, L);				
				float4 col = tex2D(_MainTex, float2(NdL, 0.5));
				float3 V = normalize(i.viewDir);
				float edgeValue = smoothstep(contourThreshold, contourThreshold + contourFalloff, dot(N,V));
				col.rgb *= edgeValue;
				return col;
			}		
			ENDCG
		}
	}
}
