﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
[RequireComponent(typeof(MeshFilter))]
public class ScreenSpaceStrokes : MonoBehaviour {

    Vector3[] meshVertices;
    Vector2[] meshUVs;
    int[] meshTriangles;
    int lastStrokeCount = 0;
    public int strokeCount = 0;    
    public Vector2 strokeSize = new Vector2(0.01f, 0.005f);
    Vector2 lastStrokeSize = new Vector2(0.01f, 0.005f);
    public Color strokeColor = new Color(0,0,0,1);


    void Start () {
        Mesh mesh = new Mesh();
        GetComponent<MeshFilter>().mesh = mesh;
        mesh.indexFormat = UnityEngine.Rendering.IndexFormat.UInt32;
        mesh.vertices = meshVertices;
        mesh.uv = meshUVs;
        mesh.triangles = meshTriangles;
    }
	
	// Update is called once per frame
	void Update () {
		if(strokeCount != lastStrokeCount || strokeSize != lastStrokeSize)
        {
            Mesh mesh = GetComponent<MeshFilter>().sharedMesh;
            mesh.Clear();

            lastStrokeCount = strokeCount;
            lastStrokeSize = strokeSize;

            meshVertices = new Vector3[strokeCount * 4];
            meshUVs = new Vector2[strokeCount * 4];
            meshTriangles = new int[strokeCount * 6];

            for(int i = 0; i < strokeCount; ++i)
            {
                float randX = Random.Range(-1.0f, 1.0f);
                float randY = Random.Range(-1.0f, 1.0f);
                float priority = Random.Range(0.0f, 1.0f);

                meshVertices[i * 4    ] = new Vector3(randX, randY, priority); meshUVs[i * 4    ] = new Vector2(0, 0);
                meshVertices[i * 4 + 1] = new Vector3(randX, randY, priority); meshUVs[i * 4 + 1] = new Vector2(1, 0);
                meshVertices[i * 4 + 2] = new Vector3(randX, randY, priority); meshUVs[i * 4 + 2] = new Vector2(1, 1);
                meshVertices[i * 4 + 3] = new Vector3(randX, randY, priority); meshUVs[i * 4 + 3] = new Vector2(0, 1);

                meshTriangles[i * 6    ] = i * 4 + 3;
                meshTriangles[i * 6 + 1] = i * 4 + 1;
                meshTriangles[i * 6 + 2] = i * 4;
                meshTriangles[i * 6 + 3] = i * 4 + 3;
                meshTriangles[i * 6 + 4] = i * 4 + 2;
                meshTriangles[i * 6 + 5] = i * 4 + 1;
            }

            mesh.vertices = meshVertices;
            mesh.uv = meshUVs;
            mesh.triangles = meshTriangles;
        }

        Material material = GetComponent<MeshRenderer>().sharedMaterial;
        material.SetVector("strokeSize", new Vector4(strokeSize.x * 0.01f, strokeSize.y * 0.01f, 0, 0));
        material.SetColor("strokeColor", strokeColor);

    }
    
}
