﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class TonalArtMaps : MonoBehaviour {

    Texture3D strokeMap;
    public Vector2 strokeTiling = new Vector2(1, 1);
     void Start()
    {
        Camera camera = GetComponent<Camera>();
        Shader shader = Shader.Find("NPR/TexturedStrokes");
        camera.SetReplacementShader(shader, "");

        int Res = 256; //highest resolution
        int LayerDuplicates = 8;
        strokeMap = new Texture3D(Res, Res, 5 * LayerDuplicates, TextureFormat.RGBA32, true);

        for (int mips = 0; mips < 4; mips++)
        {
            int offset = 0;
            Color32[] allPixels = new Color32[Res * Res * 5 * LayerDuplicates];
            for (int layers = 0; layers < 5; layers++)
            {
                string textureName = "Assets/Lab06/TAM/" + (5 - layers) + "_" + (mips + 1) + ".jpg";
                Texture2D texture = (Texture2D)AssetDatabase.LoadAssetAtPath(textureName, typeof(Texture2D));
                Color32[] texPixels = texture.GetPixels32(0);
                for (int dup = 0; dup < LayerDuplicates; ++dup)
                {
                    System.Array.Copy(texPixels, 0, allPixels, offset, Res * Res);
                    offset += Res * Res;
                }
            }
            Color32[] spixels = strokeMap.GetPixels32(mips);
            strokeMap.SetPixels32(allPixels, mips);
            Res /= 2;
            LayerDuplicates /= 2;
        }

        for (int mips = 4; mips < 9; mips++)
        {
            Color32[] pixels = strokeMap.GetPixels32(mips);
            for (int p = 0; p < pixels.Length; ++p)
                pixels[p].r = pixels[p].g = pixels[p].b = pixels[p].a = 255;
            strokeMap.SetPixels32(pixels, mips);
        }

        strokeMap.wrapModeU = TextureWrapMode.Repeat;
        strokeMap.wrapModeV = TextureWrapMode.Repeat;
        strokeMap.wrapModeW = TextureWrapMode.Clamp;
        strokeMap.filterMode = FilterMode.Trilinear;
        strokeMap.mipMapBias = -0.5f;
        strokeMap.Apply(false);
    }

    // Update is called once per frame
    void Update()
    {
        Shader.SetGlobalTexture("_StrokeMap", strokeMap);
        Shader.SetGlobalVector("textureScale", strokeTiling);
    }
}
